'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  return {
    visitor: {
      ImportDeclaration: {
        enter: function enter(path) {
          // Is the styled-components import!
          if (path.node.source.value === 'styled-components') {
            // If the default is imported it's at defaultImport[0], otherwise defaultImport is empty
            var defaultImport = path.node.specifiers.filter(function (specifier) {
              return t.isImportDefaultSpecifier(specifier);
            });
            if (defaultImport.length > 0) {
              // Save the imported name
              importedVariableName = defaultImport[0].local.name;
            }
          }
        }
      },
      TaggedTemplateExpression: {
        enter: function enter(path, _ref2) {
          var opts = _ref2.opts;

          var addDisplayName = opts.displayName === undefined || opts.displayName === null ? true : opts.displayName;
          var addIdentifier = opts.ssr === undefined || opts.ssr === null ? true : opts.ssr;
          var tag = path.node.tag;

          if (!isStyled(tag)) return;

          var displayName = void 0;

          path.find(function (path) {
            // const X = styled
            if (path.isAssignmentExpression()) {
              displayName = path.node.left;
              // const X = { Y: styled }
            } else if (path.isObjectProperty()) {
              displayName = path.node.key;
              // let X; X = styled
            } else if (path.isVariableDeclarator()) {
              displayName = path.node.id;
            } else if (path.isStatement()) {
              // we've hit a statement, we should stop crawling up
              return true;
            }

            // we've got an displayName (if we need it) no need to continue
            if (displayName) return true;
          });

          // foo.bar -> bar
          if (t.isMemberExpression(displayName)) {
            displayName = displayName.property;
          }

          // Get target
          var target = (0, _getTarget2.default)(path.node.tag);

          // identifiers are the only thing we can reliably get a name from
          if (!t.isIdentifier(displayName)) {
            displayName = undefined;
          } else {
            displayName = displayName.name;
          }

          id++;
          // Prefix the identifier with a character if no displayName exists because CSS classes cannot start with a number
          var identifier = (displayName || 's') + '-' + (0, _hash2.default)('' + id + displayName);
          // Put together the final code again
          var call = buildStyledCall({
            STYLED: t.identifier(importedVariableName),
            TARGET: target,
            DISPLAYNAME: addDisplayName && displayName && t.stringLiteral(displayName) || t.identifier('undefined'),
            IDENTIFIER: addIdentifier && identifier && t.stringLiteral(identifier) || t.identifier('undefined')
          });
          // Put together the styled call with the template literal
          // to get the finished styled({ })`` form! 🎉
          path.node.tag = call.expression;
        }
      }
    }
  };
};

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

var _hash = require('./utils/hash');

var _hash2 = _interopRequireDefault(_hash);

var _getTarget = require('./utils/get-target');

var _getTarget2 = _interopRequireDefault(_getTarget);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var buildStyledCall = (0, _babelTemplate2.default)('STYLED({\n  target: TARGET,\n  displayName: DISPLAYNAME,\n  identifier: IDENTIFIER\n})');

// Default imported variable name to "styled", adjust based on import below
var importedVariableName = 'styled';

var isStyled = function isStyled(tag) {
  return tag.object && tag.object.name === importedVariableName || tag.callee && tag.callee.name === importedVariableName;
};

var id = 0;